@@ -23,7 +23,7 @@ import typing
 
 from PyQt5.QtCore import Qt, QModelIndex, QAbstractItemModel
 
-from qutebrowser.utils import log, qtutils
+from qutebrowser.utils import log, qtutils, utils
 from qutebrowser.api import cmdutils
 
 
@@ -153,8 +153,8 @@ class CompletionModel(QAbstractItemModel):
 
     def columnCount(self, parent=QModelIndex()):
         """Override QAbstractItemModel::columnCount."""
-        # pylint: disable=unused-argument
-        return 3
+        utils.unused(parent)
+        return len(self.column_widths)
 
     def canFetchMore(self, parent):
         """Override to forward the call to the categories."""
@@ -183,11 +183,13 @@ class CompletionModel(QAbstractItemModel):
         # WORKAROUND:
         # layoutChanged is broken in PyQt 5.7.1, so we must use metaObject
         # https://www.riverbankcomputing.com/pipermail/pyqt/2017-January/038483.html
-        self.metaObject().invokeMethod(self,  # type: ignore
-                                       "layoutAboutToBeChanged")
+        meta = self.metaObject()
+        meta.invokeMethod(self,  # type: ignore[misc, call-overload]
+                          "layoutAboutToBeChanged")
         for cat in self._categories:
             cat.set_pattern(pattern)
-        self.metaObject().invokeMethod(self, "layoutChanged")  # type: ignore
+        meta.invokeMethod(self,  # type: ignore[misc, call-overload]
+                          "layoutChanged")
 
     def first_item(self):
         """Return the index of the first child (non-category) in the model."""
