@@ -28,7 +28,7 @@ import pathlib
 
 import pytest
 import hypothesis
-from PyQt5.QtCore import qVersion, PYQT_VERSION
+from PyQt5.QtCore import PYQT_VERSION
 
 pytest.register_assert_rewrite('helpers')
 
@@ -48,10 +48,12 @@ _qute_scheme_handler = None
 
 
 # Set hypothesis settings
-hypothesis.settings.register_profile('default',
-                                     hypothesis.settings(deadline=600))
-hypothesis.settings.register_profile('ci',
-                                     hypothesis.settings(deadline=None))
+hypothesis.settings.register_profile(
+    'default', hypothesis.settings(deadline=600))
+hypothesis.settings.register_profile(
+    'ci', hypothesis.settings(
+        deadline=None,
+        suppress_health_check=[hypothesis.HealthCheck.too_slow]))
 hypothesis.settings.load_profile('ci' if testutils.ON_CI else 'default')
 
 
@@ -197,12 +199,10 @@ def pytest_ignore_collect(path):
 
 @pytest.fixture(scope='session')
 def qapp_args():
-    """Make QtWebEngine unit tests run on Qt 5.7.1.
-
-    See https://github.com/qutebrowser/qutebrowser/issues/3163
-    """
-    if qVersion() == '5.7.1':
-        return [sys.argv[0], '--disable-seccomp-filter-sandbox']
+    """Make QtWebEngine unit tests run on older Qt versions + newer kernels."""
+    seccomp_args = testutils.seccomp_args(qt_flag=False)
+    if seccomp_args:
+        return [sys.argv[0]] + seccomp_args
     return []
 
 
@@ -224,8 +224,8 @@ def pytest_addoption(parser):
 
 def pytest_configure(config):
     webengine_arg = config.getoption('--qute-bdd-webengine')
-    webengine_env = os.environ.get('QUTE_BDD_WEBENGINE', '')
-    config.webengine = bool(webengine_arg or webengine_env)
+    webengine_env = os.environ.get('QUTE_BDD_WEBENGINE', 'false')
+    config.webengine = webengine_arg or webengine_env == 'true'
     # Fail early if QtWebEngine is not available
     if config.webengine:
         import PyQt5.QtWebEngineWidgets
@@ -234,11 +234,6 @@ def pytest_configure(config):
 
 @pytest.fixture(scope='session', autouse=True)
 def check_display(request):
-    if (not request.config.getoption('--no-xvfb') and
-            'QUTE_BUILDBOT' in os.environ and
-            request.config.xvfb is not None):
-        raise Exception("Xvfb is running on buildbot!")
-
     if utils.is_linux and not os.environ.get('DISPLAY', ''):
         raise Exception("No display and no Xvfb available!")
 
@@ -294,8 +289,12 @@ def apply_fake_os(monkeypatch, request):
 
 @pytest.fixture(scope='session', autouse=True)
 def check_yaml_c_exts():
-    """Make sure PyYAML C extensions are available on CI."""
-    if 'CI' in os.environ:
+    """Make sure PyYAML C extensions are available on CI.
+
+    Not available yet with a nightly Python, see:
+    https://github.com/yaml/pyyaml/issues/416
+    """
+    if 'CI' in os.environ and sys.version_info[:2] != (3, 10):
         from yaml import CLoader
 
 
@@ -308,3 +307,15 @@ def pytest_runtest_makereport(item, call):
     outcome = yield
     rep = outcome.get_result()
     setattr(item, "rep_" + rep.when, rep)
+
+
+@pytest.hookimpl(hookwrapper=True)
+def pytest_terminal_summary(terminalreporter):
+    """Group benchmark results on CI."""
+    if testutils.ON_CI:
+        terminalreporter.write_line(
+            testutils.gha_group_begin('Benchmark results'))
+        yield
+        terminalreporter.write_line(testutils.gha_group_end())
+    else:
+        yield
