@@ -481,9 +481,10 @@ class TabbedBrowserStub(QObject):
     def __init__(self, parent=None):
         super().__init__(parent)
         self.widget = TabWidgetStub()
-        self.shutting_down = False
+        self.is_shutting_down = False
         self.loaded_url = None
         self.cur_url = None
+        self.undo_stack = None
 
     def on_tab_close_requested(self, idx):
         del self.widget.tabs[idx]
