@@ -26,9 +26,17 @@ import pprint
 import os.path
 import contextlib
 import pathlib
+import importlib.util
+import importlib.machinery
 
 import pytest
 
+from PyQt5.QtCore import qVersion
+try:
+    from PyQt5.QtWebEngine import PYQT_WEBENGINE_VERSION_STR
+except ImportError:
+    PYQT_WEBENGINE_VERSION_STR = None
+
 from qutebrowser.utils import qtutils, log
 
 ON_CI = 'CI' in os.environ
@@ -123,6 +131,24 @@ def _partial_compare_eq(val1, val2, *, indent):
     return PartialCompareOutcome("{!r} != {!r}".format(val1, val2))
 
 
+def gha_group_begin(name):
+    """Get a string to begin a GitHub Actions group.
+
+    Should only be called on CI.
+    """
+    assert ON_CI
+    return '::group::' + name
+
+
+def gha_group_end():
+    """Get a string to end a GitHub Actions group.
+
+    Should only be called on CI.
+    """
+    assert ON_CI
+    return '::endgroup::'
+
+
 def partial_compare(val1, val2, *, indent=0):
     """Do a partial comparison between the given values.
 
@@ -132,6 +158,9 @@ def partial_compare(val1, val2, *, indent=0):
 
     This happens recursively.
     """
+    if ON_CI and indent == 0:
+        print(gha_group_begin('Comparison'))
+
     print_i("Comparing", indent)
     print_i(pprint.pformat(val1), indent + 1)
     print_i("|---- to ----", indent)
@@ -163,6 +192,10 @@ def partial_compare(val1, val2, *, indent=0):
         print_i("|======= Comparing via ==", indent)
         outcome = _partial_compare_eq(val1, val2, indent=indent)
     print_i("---> {}".format(outcome), indent)
+
+    if ON_CI and indent == 0:
+        print(gha_group_end())
+
     return outcome
 
 
@@ -211,3 +244,60 @@ def ignore_bs4_warning():
 def blocked_hosts():
     path = os.path.join(abs_datapath(), 'blocked-hosts.gz')
     yield from io.TextIOWrapper(gzip.open(path), encoding='utf-8')
+
+
+def seccomp_args(qt_flag):
+    """Get necessary flags to disable the seccomp BPF sandbox.
+
+    This is needed for some QtWebEngine setups, with older Qt versions but
+    newer kernels.
+
+    Args:
+        qt_flag: Add a '--qt-flag' argument.
+    """
+    affected_versions = set()
+    for base, patch_range in [
+            ## seccomp-bpf failure in syscall 0281
+            ## https://github.com/qutebrowser/qutebrowser/issues/3163
+            # 5.7.1
+            ('5.7', [1]),
+
+            ## seccomp-bpf failure in syscall 0281 (clock_nanosleep)
+            ## https://bugreports.qt.io/browse/QTBUG-81313
+            # 5.11.0 to 5.11.3 (inclusive)
+            ('5.11', range(0, 4)),
+            # 5.12.0 to 5.12.7 (inclusive)
+            ('5.12', range(0, 8)),
+            # 5.13.0 to 5.13.2 (inclusive)
+            ('5.13', range(0, 3)),
+            # 5.14.0
+            ('5.14', [0]),
+    ]:
+        for patch in patch_range:
+            affected_versions.add('{}.{}'.format(base, patch))
+
+    version = (PYQT_WEBENGINE_VERSION_STR
+               if PYQT_WEBENGINE_VERSION_STR is not None
+               else qVersion())
+    if version in affected_versions:
+        disable_arg = 'disable-seccomp-filter-sandbox'
+        return ['--qt-flag', disable_arg] if qt_flag else ['--' + disable_arg]
+
+    return []
+
+
+def import_userscript(name):
+    """Import a userscript via importlib.
+
+    This is needed because userscripts don't have a .py extension and violate
+    Python's module naming convention.
+    """
+    repo_root = pathlib.Path(__file__).resolve().parents[2]
+    script_path = repo_root / 'misc' / 'userscripts' / name
+    module_name = name.replace('-', '_')
+    loader = importlib.machinery.SourceFileLoader(
+        module_name, str(script_path))
+    spec = importlib.util.spec_from_loader(module_name, loader)
+    module = importlib.util.module_from_spec(spec)
+    spec.loader.exec_module(module)
+    return module
