@@ -22,20 +22,29 @@
 import collections
 import random
 import string
+import time
 from datetime import datetime
+from unittest import mock
 
 import pytest
-from PyQt5.QtCore import QUrl
+from PyQt5.QtCore import QUrl, QDateTime
+try:
+    from PyQt5.QtWebEngineWidgets import (
+        QWebEngineHistory, QWebEngineHistoryItem
+    )
+except ImportError:
+    pass
 
 from qutebrowser.misc import objects
 from qutebrowser.completion import completer
 from qutebrowser.completion.models import miscmodels, urlmodel, configmodel
 from qutebrowser.config import configdata, configtypes
 from qutebrowser.utils import usertypes
+from qutebrowser.mainwindow import tabbedbrowser
 
 
 def _check_completions(model, expected):
-    """Check that a model contains the expected items in any order.
+    """Check that a model contains the expected items in order.
 
     Args:
         expected: A dict of form
@@ -59,7 +68,6 @@ def _check_completions(model, expected):
             actual[catname].append((name, desc, misc))
     assert actual == expected
     # sanity-check the column_widths
-    assert len(model.column_widths) == 3
     assert sum(model.column_widths) == 100
 
 
@@ -116,7 +124,7 @@ def configdata_stub(config_stub, monkeypatch, configdata_init):
             no_autoconfig=True)),
         ('bindings.commands', configdata.Option(
             name='bindings.commands',
-            description='Default keybindings',
+            description='Custom keybindings',
             typ=configtypes.Dict(
                 keytype=configtypes.String(),
                 valtype=configtypes.Dict(
@@ -210,7 +218,8 @@ def web_history_populated(web_history):
 def info(config_stub, key_config_stub):
     return completer.CompletionInfo(config=config_stub,
                                     keyconf=key_config_stub,
-                                    win_id=0)
+                                    win_id=0,
+                                    cur_tab=None)
 
 
 def test_command_completion(qtmodeltester, cmdutils_stub, configdata_stub,
@@ -260,13 +269,38 @@ def test_help_completion(qtmodeltester, cmdutils_stub, key_config_stub,
             (':tab-close', 'Close the current tab.', ''),
         ],
         "Settings": [
-            ('aliases', 'Aliases for commands.', None),
-            ('bindings.commands', 'Default keybindings', None),
-            ('bindings.default', 'Default keybindings', None),
-            ('completion.open_categories', 'Which categories to show (in '
-             'which order) in the :open completion.', None),
-            ('content.javascript.enabled', 'Enable/Disable JavaScript', None),
-            ('url.searchengines', 'searchengines list', None),
+            (
+                'aliases',
+                'Aliases for commands.',
+                '{"q": "quit"}',
+            ),
+            (
+                'bindings.commands',
+                'Custom keybindings',
+                ('{"normal": {"<Ctrl+q>": "quit", "I": "invalid", "ZQ": "quit", '
+                 '"d": "scroll down"}}'),
+            ),
+            (
+                'bindings.default',
+                'Default keybindings',
+                '{"normal": {"<Ctrl+q>": "quit", "d": "tab-close"}}',
+            ),
+            (
+                'completion.open_categories',
+                'Which categories to show (in which order) in the :open completion.',
+                '["searchengines", "quickmarks", "bookmarks", "history"]',
+            ),
+            (
+                'content.javascript.enabled',
+                'Enable/Disable JavaScript',
+                'true'
+            ),
+            (
+                'url.searchengines',
+                'searchengines list',
+                ('{"DEFAULT": "https://duckduckgo.com/?q={}", '
+                 '"google": "https://google.com/?q={}"}'),
+            ),
         ],
     })
 
@@ -900,7 +934,7 @@ def test_setting_option_completion(qtmodeltester, config_stub,
     _check_completions(model, {
         "Options": [
             ('aliases', 'Aliases for commands.', '{"q": "quit"}'),
-            ('bindings.commands', 'Default keybindings', (
+            ('bindings.commands', 'Custom keybindings', (
                 '{"normal": {"<Ctrl+q>": "quit", "I": "invalid", '
                 '"ZQ": "quit", "d": "scroll down"}}')),
             ('completion.open_categories', 'Which categories to show (in '
@@ -924,7 +958,7 @@ def test_setting_dict_option_completion(qtmodeltester, config_stub,
     _check_completions(model, {
         "Dict options": [
             ('aliases', 'Aliases for commands.', '{"q": "quit"}'),
-            ('bindings.commands', 'Default keybindings', (
+            ('bindings.commands', 'Custom keybindings', (
                 '{"normal": {"<Ctrl+q>": "quit", "I": "invalid", '
                 '"ZQ": "quit", "d": "scroll down"}}')),
             ('url.searchengines', 'searchengines list',
@@ -1179,3 +1213,114 @@ def test_url_completion_benchmark(benchmark, info,
         model.set_pattern('ex 123')
 
     benchmark(bench)
+
+
+@pytest.fixture
+def tab_with_history(fake_web_tab, tabbed_browser_stubs, info, monkeypatch):
+    """Returns a fake tab with some fake history items."""
+    pytest.importorskip('PyQt5.QtWebEngineWidgets')
+    tab = fake_web_tab(QUrl('https://github.com'), 'GitHub', 0)
+    current_idx = 2
+    monkeypatch.setattr(
+        tab.history, 'current_idx',
+        lambda: current_idx,
+    )
+
+    history = []
+    now = time.time()
+    for url, title, ts in [
+            ("http://example.com/index", "list of things", now),
+            ("http://example.com/thing1", "thing1 detail", now+5),
+            ("http://example.com/thing2", "thing2 detail", now+10),
+            ("http://example.com/thing3", "thing3 detail", now+15),
+            ("http://example.com/thing4", "thing4 detail", now+20),
+    ]:
+        entry = mock.Mock(spec=QWebEngineHistoryItem)
+        entry.url.return_value = QUrl(url)
+        entry.title.return_value = title
+        dt = QDateTime.fromMSecsSinceEpoch(int(ts * 1000))
+        entry.lastVisited.return_value = dt
+        history.append(entry)
+    tab.history._history = mock.Mock(spec=QWebEngineHistory)
+    tab.history._history.items.return_value = history
+    monkeypatch.setattr(
+        tab.history, 'back_items',
+        lambda *_args: (
+            entry for idx, entry in enumerate(tab.history._history.items())
+            if idx < current_idx
+        ),
+    )
+    monkeypatch.setattr(
+        tab.history, 'forward_items',
+        lambda *_args: (
+            entry for idx, entry in enumerate(tab.history._history.items())
+            if idx > current_idx
+        ),
+    )
+
+    tabbed_browser_stubs[0].widget.tabs = [tab]
+    tabbed_browser_stubs[0].widget.current_index = 0
+
+    info.cur_tab = tab
+    return tab
+
+
+def test_back_completion(tab_with_history, info):
+    """Test back tab history completion."""
+    model = miscmodels.back(info=info)
+    model.set_pattern('')
+
+    _check_completions(model, {
+        "History": [
+            ("1", "http://example.com/thing1", "thing1 detail"),
+            ("0", "http://example.com/index", "list of things"),
+        ],
+    })
+
+
+def test_forward_completion(tab_with_history, info):
+    """Test forward tab history completion."""
+    model = miscmodels.forward(info=info)
+    model.set_pattern('')
+
+    _check_completions(model, {
+        "History": [
+            ("3", "http://example.com/thing3", "thing3 detail"),
+            ("4", "http://example.com/thing4", "thing4 detail"),
+        ],
+    })
+
+
+def test_undo_completion(tabbed_browser_stubs, info):
+    """Test :undo completion."""
+    entry1 = tabbedbrowser._UndoEntry(url=QUrl('https://example.org/'),
+                                      history=None, index=None, pinned=None,
+                                      created_at=datetime(2020, 1, 1))
+    entry2 = tabbedbrowser._UndoEntry(url=QUrl('https://example.com/'),
+                                      history=None, index=None, pinned=None,
+                                      created_at=datetime(2020, 1, 2))
+    entry3 = tabbedbrowser._UndoEntry(url=QUrl('https://example.net/'),
+                                      history=None, index=None, pinned=None,
+                                      created_at=datetime(2020, 1, 2))
+
+    # Most recently closed is at the end
+    tabbed_browser_stubs[0].undo_stack = [
+        [entry1],
+        [entry2, entry3],
+    ]
+
+    model = miscmodels.undo(info=info)
+    model.set_pattern('')
+
+    # Most recently closed is at the top, indices are used like "-x" for the
+    # undo stack.
+    _check_completions(model, {
+        "Closed tabs": [
+            ("1",
+             "https://example.com/, https://example.net/",
+             "2020-01-02 00:00"),
+            ("2",
+             "https://example.org/",
+             "2020-01-01 00:00"),
+        ],
+    })
