@@ -458,11 +458,11 @@ class TestKeySequence:
          Qt.ControlModifier | Qt.ShiftModifier),
         (Qt.ShiftModifier, Qt.ShiftModifier),
     ])
-    def test_fake_mac(self, fake_keyevent, modifiers, expected):
+    def test_fake_mac(self, modifiers, expected):
         """Make sure Control/Meta are swapped with a simulated Mac."""
         seq = keyutils.KeySequence()
-        event = fake_keyevent(key=Qt.Key_A, modifiers=modifiers)
-        new = seq.append_event(event)
+        info = keyutils.KeyInfo(key=Qt.Key_A, modifiers=modifiers)
+        new = seq.append_event(info.to_event())
         assert new[0] == keyutils.KeyInfo(Qt.Key_A, expected)
 
     @pytest.mark.parametrize('key', [Qt.Key_unknown, 0x0])
@@ -579,28 +579,6 @@ def test_is_printable(key, printable):
     assert keyutils.is_special(key, Qt.NoModifier) != printable
 
 
-@pytest.mark.parametrize('key, modifiers, special', [
-    (Qt.Key_Escape, Qt.NoModifier, True),
-    (Qt.Key_Escape, Qt.ShiftModifier, True),
-    (Qt.Key_Escape, Qt.ControlModifier, True),
-    (Qt.Key_X, Qt.ControlModifier, True),
-    (Qt.Key_X, Qt.NoModifier, False),
-    (Qt.Key_2, Qt.NoModifier, False),
-
-    # Keypad should not reset hint keychain - see #3735
-    (Qt.Key_2, Qt.KeypadModifier, False),
-
-    # Modifiers should not reset hint keychain - see #4264
-    (Qt.Key_Shift, Qt.ShiftModifier, False),
-    (Qt.Key_Control, Qt.ControlModifier, False),
-    (Qt.Key_Alt, Qt.AltModifier, False),
-    (Qt.Key_Meta, Qt.MetaModifier, False),
-    (Qt.Key_Mode_switch, Qt.GroupSwitchModifier, False),
-])
-def test_is_special_hint_mode(key, modifiers, special):
-    assert keyutils.is_special_hint_mode(key, modifiers) == special
-
-
 @pytest.mark.parametrize('key, modifiers, special', [
     (Qt.Key_Escape, Qt.NoModifier, True),
     (Qt.Key_Escape, Qt.ShiftModifier, True),
