@@ -18,10 +18,9 @@
 # along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
 
 import logging
-from unittest import mock
 
 from PyQt5.QtCore import Qt, QSize
-from PyQt5.QtWidgets import QApplication, QWidget
+from PyQt5.QtWidgets import QWidget
 import pytest
 
 from qutebrowser.misc import miscwidgets
@@ -41,19 +40,6 @@ class TestCommandLineEdit:
         assert cmd_edit.text() == ''
         yield cmd_edit
 
-    @pytest.fixture
-    def mock_clipboard(self, mocker):
-        """Fixture to mock QApplication.clipboard.
-
-        Return:
-            The mocked QClipboard object.
-        """
-        mocker.patch.object(QApplication, 'clipboard')
-        clipboard = mock.MagicMock()
-        clipboard.supportsSelection.return_value = True
-        QApplication.clipboard.return_value = clipboard
-        return clipboard
-
     def test_position(self, qtbot, cmd_edit):
         """Test cursor position based on the prompt."""
         qtbot.keyClicks(cmd_edit, ':hello')
@@ -158,7 +144,8 @@ class TestInspectorSplitter:
 
     @pytest.fixture
     def splitter(self, qtbot, fake_webview):
-        inspector_splitter = miscwidgets.InspectorSplitter(fake_webview)
+        inspector_splitter = miscwidgets.InspectorSplitter(
+            win_id=0, main_webview=fake_webview)
         qtbot.add_widget(inspector_splitter)
         return inspector_splitter
 
@@ -171,6 +158,11 @@ class TestInspectorSplitter:
         splitter.show()
         splitter.resize(800, 600)
 
+    def test_cycle_focus_no_inspector(self, splitter):
+        with pytest.raises(inspector.Error,
+                           match='No inspector inside main window'):
+            splitter.cycle_focus()
+
     @pytest.mark.parametrize(
         'position, orientation, inspector_idx, webview_idx', [
             (inspector.Position.left, Qt.Horizontal, 0, 1),
@@ -192,6 +184,14 @@ class TestInspectorSplitter:
 
         assert splitter.orientation() == orientation
 
+    def test_cycle_focus_hidden_inspector(self, splitter, fake_inspector):
+        splitter.set_inspector(fake_inspector, inspector.Position.right)
+        splitter.show()
+        fake_inspector.hide()
+        with pytest.raises(inspector.Error,
+                           match='No inspector inside main window'):
+            splitter.cycle_focus()
+
     @pytest.mark.parametrize(
         'config, width, height, position, expected_size', [
             # No config but enough big window
