@@ -23,7 +23,6 @@ The tests are mostly inspired by Chromium's:
 https://cs.chromium.org/chromium/src/extensions/common/url_pattern_unittest.cc
 
 Currently not tested:
-- The match_effective_tld attribute as it doesn't exist yet.
 - Nested filesystem:// URLs as we don't have those.
 - Unicode matching because QUrl doesn't like those URLs.
 - Any other features we don't need, such as .GetAsString() or set operations.
@@ -41,33 +40,52 @@ from qutebrowser.utils import urlmatch
 
 
 @pytest.mark.parametrize('pattern, error', [
-    # Chromium: PARSE_ERROR_MISSING_SCHEME_SEPARATOR
+    ### Chromium: kMissingSchemeSeparator
+    ## TEST(ExtensionURLPatternTest, ParseInvalid)
     # ("http", "No scheme given"),
     ("http:", "Invalid port: Port is empty"),
     ("http:/", "Invalid port: Port is empty"),
     ("about://", "Pattern without path"),
     ("http:/bar", "Invalid port: Port is empty"),
 
-    # Chromium: PARSE_ERROR_EMPTY_HOST
+    ### Chromium: kEmptyHost
+    ## TEST(ExtensionURLPatternTest, ParseInvalid)
     ("http://", "Pattern without host"),
     ("http:///", "Pattern without host"),
-    ("http:// /", "Pattern without host"),
     ("http://:1234/", "Pattern without host"),
+    ("http://*./", "Pattern without host"),
+    ## TEST(ExtensionURLPatternTest, IPv6Patterns)
+    ("http://[]:8888/*", "Pattern without host"),
 
-    # Chromium: PARSE_ERROR_EMPTY_PATH
+    ### Chromium: kEmptyPath
+    ## TEST(ExtensionURLPatternTest, ParseInvalid)
     # We deviate from Chromium and allow this for ease of use
     # ("http://bar", "..."),
 
-    # Chromium: PARSE_ERROR_INVALID_HOST
+    ### Chromium: kInvalidHost
+    ## TEST(ExtensionURLPatternTest, ParseInvalid)
     ("http://\0www/", "May not contain NUL byte"),
+    ## TEST(ExtensionURLPatternTest, IPv6Patterns)
+    # No closing bracket (`]`).
+    ("http://[2607:f8b0:4005:805::200e/*", "Invalid IPv6 URL"),
+    # Two closing brackets (`]]`).
+    pytest.param("http://[2607:f8b0:4005:805::200e]]/*", "Invalid IPv6 URL", marks=pytest.mark.xfail(reason="https://bugs.python.org/issue34360")),
+    # Two open brackets (`[[`).
+    ("http://[[2607:f8b0:4005:805::200e]/*", r"""Expected '\]' to match '\[' in hostname; source was "\[2607:f8b0:4005:805::200e"; host = """""),
+    # Too few colons in the last chunk.
+    ("http://[2607:f8b0:4005:805:200e]/*", 'Invalid IPv6 address; source was "2607:f8b0:4005:805:200e"; host = ""'),
+    # Non-hex piece.
+    ("http://[2607:f8b0:4005:805:200e:12:bogus]/*", 'Invalid IPv6 address; source was "2607:f8b0:4005:805:200e:12:bogus"; host = ""'),
 
-    # Chromium: PARSE_ERROR_INVALID_HOST_WILDCARD
+    ### Chromium: kInvalidHostWildcard
+    ## TEST(ExtensionURLPatternTest, ParseInvalid)
     ("http://*foo/bar", "Invalid host wildcard"),
     ("http://foo.*.bar/baz", "Invalid host wildcard"),
     ("http://fo.*.ba:123/baz", "Invalid host wildcard"),
-    ("http://foo.*/bar", "TLD wildcards are not implemented yet"),
+    ("http://foo.*/bar", "Invalid host wildcard"),
 
-    # Chromium: PARSE_ERROR_INVALID_PORT
+    ### Chromium: kInvalidPort
+    ## TEST(ExtensionURLPatternTest, Ports)
     ("http://foo:/", "Invalid port: Port is empty"),
     ("http://*.foo:/", "Invalid port: Port is empty"),
     ("http://foo:com/", "Invalid port: .* 'com'"),
@@ -78,18 +96,16 @@ from qutebrowser.utils import urlmatch
                      reason="Doesn't show an error on Python 3.5")),
     ("http://foo:80:80/monkey", "Invalid port: .* '80:80'"),
     ("chrome://foo:1234/bar", "Ports are unsupported with chrome scheme"),
+    # No port specified, but port separator.
+    ("http://[2607:f8b0:4005:805::200e]:/*", "Invalid port: Port is empty"),
 
-    # Additional tests
+    ### Additional tests
     ("http://[", "Invalid IPv6 URL"),
-    ("http://[fc2e:bb88::edac]:", "Invalid port: Port is empty"),
     ("http://[fc2e::bb88::edac]", 'Invalid IPv6 address; source was "fc2e::bb88::edac"; host = ""'),
     ("http://[fc2e:0e35:bb88::edac:fc2e:0e35:bb88:edac]", 'Invalid IPv6 address; source was "fc2e:0e35:bb88::edac:fc2e:0e35:bb88:edac"; host = ""'),
     ("http://[fc2e:0e35:bb88:af:edac:fc2e:0e35:bb88:edac]", 'Invalid IPv6 address; source was "fc2e:0e35:bb88:af:edac:fc2e:0e35:bb88:edac"; host = ""'),
     ("http://[127.0.0.1:fc2e::bb88:edac]", r'Invalid IPv6 address; source was "127\.0\.0\.1:fc2e::bb88:edac'),
-    ("http://[]:20", "Pattern without host"),
     ("http://[fc2e::bb88", "Invalid IPv6 URL"),
-    ("http://[[fc2e::bb88:edac]", r"""Expected '\]' to match '\[' in hostname; source was "\[fc2e::bb88:edac"; host = """""),
-    pytest.param("http://[fc2e::bb88:edac]]", "Invalid IPv6 URL", marks=pytest.mark.xfail(reason="https://bugs.python.org/issue34360")),
     ("http://[fc2e:bb88:edac]", 'Invalid IPv6 address; source was "fc2e:bb88:edac"; host = ""'),
     ("http://[fc2e:bb88:edac::z]", 'Invalid IPv6 address; source was "fc2e:bb88:edac::z"; host = ""'),
     ("http://[fc2e:bb88:edac::2]:2a2", "Invalid port: .* '2a2'"),
@@ -100,7 +116,23 @@ def test_invalid_patterns(pattern, error):
         urlmatch.UrlPattern(pattern)
 
 
+@pytest.mark.parametrize('host', ['.', ' ', ' .', '. ', '. .', '. . .', ' . '])
+def test_whitespace_hosts(host):
+    """Test that whitespace dot hosts are invalid.
+
+    This is a deviation from Chromium.
+    """
+    template = 'https://{}/*'
+    url = QUrl(template.format(host))
+    assert not url.isValid()
+
+    with pytest.raises(urlmatch.ParseError,
+                       match='Invalid host|Pattern without host'):
+        urlmatch.UrlPattern(template.format(host))
+
+
 @pytest.mark.parametrize('pattern, port', [
+    ## TEST(ExtensionURLPatternTest, Ports)
     ("http://foo:1234/", 1234),
     ("http://foo:1234/bar", 1234),
     ("http://*.foo:1234/", 1234),
@@ -109,13 +141,10 @@ def test_invalid_patterns(pattern, error):
     ("http://*:*/", None),
     ("http://foo:*/", None),
     ("file://foo:1234/bar", None),
-
     # Port-like strings in the path should not trigger a warning.
     ("http://*/:1234", None),
     ("http://*.foo/bar:1234", None),
     ("http://foo/bar:1234/path", None),
-    # We don't implement ALLOW_WILDCARD_FOR_EFFECTIVE_TLD yet.
-    # ("http://*.foo.*/:1234", None),
 ])
 def test_port(pattern, port):
     up = urlmatch.UrlPattern(pattern)
@@ -151,6 +180,8 @@ def test_lightweight_patterns(pattern, scheme, host, path):
 
 class TestMatchAllPagesForGivenScheme:
 
+    """Based on TEST(ExtensionURLPatternTest, Match1)."""
+
     @pytest.fixture
     def up(self):
         return urlmatch.UrlPattern("http://*/*")
@@ -164,12 +195,13 @@ class TestMatchAllPagesForGivenScheme:
 
     @pytest.mark.parametrize('url, expected', [
         ("http://google.com", True),
-        ("http://google.com:80", True),
-        ("http://google.com.", True),
         ("http://yahoo.com", True),
         ("http://google.com/foo", True),
         ("https://google.com", False),
         ("http://74.125.127.100/search", True),
+        # Additional tests
+        ("http://google.com:80", True),
+        ("http://google.com.", True),
         ("http://[fc2e:0e35:bb88::edac]", True),
         ("http://[fc2e:e35:bb88::edac]", True),
         ("http://[fc2e:e35:bb88::127.0.0.1]", True),
@@ -181,6 +213,8 @@ class TestMatchAllPagesForGivenScheme:
 
 class TestMatchAllDomains:
 
+    """Based on TEST(ExtensionURLPatternTest, Match2)."""
+
     @pytest.fixture
     def up(self):
         return urlmatch.UrlPattern("https://*/foo*")
@@ -204,6 +238,8 @@ class TestMatchAllDomains:
 
 class TestMatchSubdomains:
 
+    """Based on TEST(ExtensionURLPatternTest, Match3)."""
+
     @pytest.fixture
     def up(self):
         return urlmatch.UrlPattern("http://*.google.com/foo*bar")
@@ -229,6 +265,8 @@ class TestMatchSubdomains:
 
 class TestMatchGlobEscaping:
 
+    """Based on TEST(ExtensionURLPatternTest, Match5)."""
+
     @pytest.fixture
     def up(self):
         return urlmatch.UrlPattern(r"file:///foo-bar\*baz")
@@ -241,6 +279,7 @@ class TestMatchGlobEscaping:
         assert up._path == r'/foo-bar\*baz'
 
     @pytest.mark.parametrize('url, expected', [
+        ## TEST(ExtensionURLPatternTest, Match5)
         # We use - instead of ? so it doesn't get treated as query
         (r"file:///foo-bar\hellobaz", True),
         (r"file:///fooXbar\hellobaz", False),
@@ -251,9 +290,12 @@ class TestMatchGlobEscaping:
 
 class TestMatchIpAddresses:
 
+    """Based on TEST(ExtensionURLPatternTest, Match6/7)."""
+
     @pytest.mark.parametrize('pattern, host, match_subdomains', [
         ("http://127.0.0.1/*", "127.0.0.1", False),
         ("http://*.0.0.1/*", "0.0.1", True),
+        ## Others
         ("http://[::1]/*", "::1", False),
         ("http://[0::1]/*", "::1", False),
         ("http://[::01]/*", "::1", False),
@@ -277,8 +319,13 @@ class TestMatchIpAddresses:
         assert up.matches(QUrl("http://127.0.0.1")) == expected
 
 
+## FIXME Missing TEST(ExtensionURLPatternTest, Match8) (unicode)?
+
+
 class TestMatchChromeUrls:
 
+    """Based on TEST(ExtensionURLPatternTest, Match9/10)."""
+
     @pytest.fixture
     def up(self):
         return urlmatch.UrlPattern("chrome://favicon/*")
@@ -301,6 +348,8 @@ class TestMatchChromeUrls:
 
 class TestMatchAnything:
 
+    """Based on TEST(ExtensionURLPatternTest, Match10/11)."""
+
     @pytest.fixture(params=['*://*/*', '*://*:*/*', '<all_urls>', '*://*'])
     def up(self, request):
         return urlmatch.UrlPattern(request.param)
@@ -329,6 +378,7 @@ class TestMatchAnything:
         "qute://version",
         "about:blank",
         "data:text/html;charset=utf-8,<html>asdf</html>",
+        "javascript:",
     ])
     def test_urls(self, up, url):
         assert up.matches(QUrl(url))
@@ -343,11 +393,14 @@ class TestMatchAnything:
     ("data:*", "about:blank", False),
 ])
 def test_special_schemes(pattern, url, expected):
+    """Based on TEST(ExtensionURLPatternTest, Match13)."""
     assert urlmatch.UrlPattern(pattern).matches(QUrl(url)) == expected
 
 
 class TestFileScheme:
 
+    """Based on TEST(ExtensionURLPatternTest, Match14/15/16)."""
+
     @pytest.fixture(params=[
         'file:///foo*',
         'file://foo*',
@@ -378,6 +431,8 @@ class TestFileScheme:
 
 class TestMatchSpecificPort:
 
+    """Based on TEST(ExtensionURLPatternTest, Match17)."""
+
     @pytest.fixture
     def up(self):
         return urlmatch.UrlPattern("http://www.example.com:80/foo")
@@ -401,6 +456,8 @@ class TestMatchSpecificPort:
 
 class TestExplicitPortWildcard:
 
+    """Based on TEST(ExtensionURLPatternTest, Match18)."""
+
     @pytest.fixture
     def up(self):
         return urlmatch.UrlPattern("http://www.example.com:*/foo")
@@ -423,6 +480,7 @@ class TestExplicitPortWildcard:
 
 
 def test_ignore_missing_slashes():
+    """Based on TEST(ExtensionURLPatternTest, IgnoreMissingBackslashes)."""
     pattern1 = urlmatch.UrlPattern("http://www.example.com/example")
     pattern2 = urlmatch.UrlPattern("http://www.example.com/example/*")
     url1 = QUrl('http://www.example.com/example')
@@ -468,6 +526,70 @@ def test_trailing_dot_domain(pattern, url):
     assert urlmatch.UrlPattern(pattern).matches(QUrl(url))
 
 
+class TestUncanonicalizedUrl:
+
+    """Test that URLPattern properly canonicalizes uncanonicalized hosts.
+
+    Equivalent to Chromium's TEST(ExtensionURLPatternTest, UncanonicalizedUrl).
+    """
+
+    @pytest.mark.parametrize('url', [
+        'https://google.com',
+        'https://maps.google.com',
+    ])
+    def test_lowercase(self, url):
+        """Simple case: canonicalization should lowercase the host.
+
+        This is important, since gOoGle.com would never be matched in
+        practice.
+        """
+        pattern = urlmatch.UrlPattern('*://*.gOoGle.com/*')
+        assert pattern.matches(QUrl(url))
+
+    @pytest.mark.parametrize('url', [
+        'https://ɡoogle.com',
+        'https://xn--oogle-qmc.com/',
+    ])
+    def test_punycode(self, url):
+        """Trickier case: internationalization with UTF8 characters.
+
+        The first 'g' isn't actually a 'g'.
+        """
+        pattern = urlmatch.UrlPattern('https://*.ɡoogle.com/*')
+        assert pattern.matches(QUrl(url))
+
+    @pytest.mark.xfail(reason="Gets accepted by urllib.parse")
+    def test_failing_canonicalization(self):
+        """Sometimes, canonicalization can fail.
+
+        Such as here, where we have invalid unicode characters. In that case,
+        URLPattern parsing should also fail.
+
+        This fails in Chromium, but Python's urllib.parse.urlparse happily
+        tries to parse it...
+        """
+        with pytest.raises(urlmatch.ParseError):
+            urlmatch.UrlPattern('https://\xef\xb7\x90zyx.com/*')
+
+    @pytest.mark.xfail(reason="We return the original string")
+    @pytest.mark.parametrize('pattern_str, string, host', [
+        ('*://*.gOoGle.com/*',
+         '*://*.google.com/*',
+         'google.com'),
+        ('https://*.ɡoogle.com/*',
+         'https://*.xn--oogle-qmc.com/*',
+         'xn--oogle-qmc.com'),
+    ])
+    def test_str(self, pattern_str, string, host):
+        """Test that str() and .host get the canonicalized string.
+
+        Contrary to Chromium, we return the original values here.
+        """
+        pattern = urlmatch.UrlPattern(pattern_str)
+        assert str(pattern) == string
+        assert pattern.host == host
+
+
 def test_urlpattern_benchmark(benchmark):
     url = QUrl('https://www.example.com/barfoobar')
 
@@ -484,9 +606,9 @@ URL_TEXT = hst.text(alphabet=string.ascii_letters)
 @hypothesis.given(pattern=hst.builds(
     lambda *a: ''.join(a),
     # Scheme
-    hst.one_of(hst.just('*'), hst.just('http'), hst.just('file')),
+    hst.sampled_from(['*', 'http', 'file']),
     # Separator
-    hst.one_of(hst.just(':'), hst.just('://')),
+    hst.sampled_from([':', '://']),
     # Host
     hst.one_of(hst.just('*'),
                hst.builds(lambda *a: ''.join(a), hst.just('*.'), URL_TEXT),
